package service

import (
	"go/ast"
)

var (
	isPointerType = false
)

func handleTypeSpec(s *ast.TypeSpec) (code string) {
	st, ok := s.Type.(*ast.StructType)
	if ok {
		return handleClass(s.Name.String(), st)
	}

	return handleType(s)
}

func handleType(s *ast.TypeSpec) (code string) {
	code += "typedef "
	code += handleExpr(s.Type)
	code += " "
	code += handleIdentExpr(s.Name)
	code += optionallyAddArrayDetails(s)
	code += ";"

	return
}

func optionallyAddArrayDetails(s *ast.TypeSpec) (code string) {
	v, ok := s.Type.(*ast.ArrayType)
	if !ok {
		return
	}

	code += "["
	code += handleArrayLen(v)
	code += "]"
	return
}

func handleArrayLen(a *ast.ArrayType) (code string) {
	return handleExpr(a.Len)
}

func handleStructType(s *ast.StructType) (code string) {
	return handleFieldList(s.Fields)
}

func handleFieldList(l *ast.FieldList) string {
	code := ""
	for _, f := range l.List {
		code += handleField(f)
	}
	return code
}

func handleField(f *ast.Field) string {
	code := ""
	code += "  "
	code += handleIdentExpr(f.Type)
	code += " "
	nado_zapyatuyu := false
	for _, n := range f.Names {
		if nado_zapyatuyu {
			code += ","
		}
		code += handleIdentExpr(n)
		nado_zapyatuyu = true
	}
	code += ";\n"
	return code
}
